/*
 * modelnet DumpManager.h
 *
 *    Used for managing tcpdump dumping
 *
 * Copyright (c) 2006 Calvin Hubble
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *    * Neither the names of Duke University nor The University of
 *      California, San Diego, nor the names of the authors or contributors
 *      may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

// Responsible for handling information about edges as well keeping
// track of file pointers for tcpdumps
//
// Note: implementation for now simply opens a file descriptor for
// each edge, at a later time, for dealing with large topologies with
// O(1000+) edges to dump, we can create a file descriptor manager to
// help cache & manage files.

#ifndef __DUMP_MANAGER_H
#define __DUMP_MANAGER_H

// #include <hash_map>
#include <map>
#include "EdgeInfo.h"

#define TCPDUMP_BUF_SIZE 524288

using namespace std;

// typedef hash_map<int, EdgeInfo> EdgeIdMap;
// typedef hash_map<int, FILE *> EdgeFPMap;
typedef map<int, EdgeInfo> EdgeIdMap;
typedef map<int, FILE *> EdgeFPMap;


// Header for a modelnet packet 
struct modelnet_hdr {
  struct timeval tv;
  unsigned int linkId;
  unsigned int	pktLength;
  unsigned int capturedLength;
};


class DumpManager 
{
public:
  DumpManager();
  
  bool addEdge(EdgeInfo edgeInfo);
  int handleModelnetData(unsigned char* data, int maxReadAmount);
  
  void closeAllFiles();
  void flushAllFiles();
  ~DumpManager();
  

private:
  // Private Methods .. mostly to try & abstract out FP managing
  FILE* createNewDumpFile(string fileName);
  FILE* getLinkFP(int linkId);
private:
  // Private variables
  EdgeIdMap edgeIdMap;
  EdgeFPMap edgeFPMap;
};

#endif
