
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/time.h>
#include <string.h>
#include "savefile.h"

#include <time.h>


#include <boost/config.hpp>
#include <iostream>
#include <vector>
#include <map>
#include <utility>
#include <algorithm>
#include <string>
#include <boost/graph/adjacency_list.hpp>

#include <boost/graph/dijkstra_shortest_paths.hpp>


#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/dom/DOMException.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>

#include <xercesc/dom/DOMErrorHandler.hpp>

#include <signal.h>

using namespace boost;
using namespace std;

#include "mngraph.h"
#include "EdgeInfo.h"
#include "DumpManager.h"


const char PROC_DUMPS[] = "/proc/sys/modelnet/tcpdump/dumps";
const char PROC_DROP_COUNT[] = "/proc/sys/modelnet/tcpdump/dropcount";

DumpManager dumpManager;

int daemon_init(void);
static void sig_usr(int);
static void sig_int(int);



/** Used by xerces when parsing the graph **/
class EdgeHandler {
public:
    ostream& _out;  
    EdgeHandler(ostream& out=cout):_out(out){}

    void operator()(MNGraph& g, const MNGraph::Edge& e) const {
	int src, dst, idx;
	string dumpLocation;
	src = g[e].i("src");
	dst = g[e].i("dst");
	idx = g[e].i("idx");
	dumpLocation = g[e].s("tcpdump");
    
	if (dumpLocation.size() > 0)
	{
	    /* edgeIdMap[idx] = EdgeInfo(idx, src, dst, dumpLocation); */
	    dumpManager.addEdge(EdgeInfo(idx, src, dst, dumpLocation));
	    cout << "Edge idx:" << idx << " src:" << src << " dst:" << dst 
		 << " dumpLocation:" << dumpLocation << endl;
	}

    }
};




int main(const int argc, const char **argv)
{  

    // Register signal handler
    if (signal(SIGINT, sig_int) == SIG_ERR) 
    {
	cout << "Could not register handler for interrupt" << endl;
	exit(1);
    }

    // Check arguments and read in the .graph file
    const char *graphfile = argv[1];
    if (argc != 2) 
    {
	cerr << "usage: " << argv[0] << " file.model" <<endl;
	exit(1);
    }  
    cerr << "reading " << graphfile << endl;

    MNGraph * g;
    g = readgraph(graphfile);
    if (!g) 
    {
	cerr << "Error reading " << argv[1] << endl;
	exit(1);
    }  
    EdgeHandler e;
    g->for_each_edge(e);
 
    int error;
    size_t size = TCPDUMP_BUF_SIZE;  
    unsigned char *BUF = (unsigned char *) malloc (TCPDUMP_BUF_SIZE);


    int temp1, temp2;
    int fd = open(PROC_DUMPS, O_RDONLY);

    if (fd == -1) 
    {
	perror("Error opening");
	exit(1);
    }

    while(1)
    {

	size = read(fd, BUF, TCPDUMP_BUF_SIZE);
	if (error == 0)
	{
	    printf ("File was closed somehow, size: %d\n", errno, size);
	    exit (1);
	}

	int consumeCount = 0;
	int consumeDelta;
	while(consumeCount < size) 
	{
	    consumeDelta = dumpManager.handleModelnetData(BUF + consumeCount,
							  size-consumeCount);
	    if (consumeDelta == 0)
		break;
	    else
		consumeCount += consumeDelta;
	}
    }
    close(fd);
    return 0;
}

// Handing kills n stuff
static void sig_int(int signo)
{
    char droppedPackets[1024];
    int fd;
    int readAmount;

    fd = open(PROC_DROP_COUNT, O_RDONLY);
    readAmount = read(fd, droppedPackets, 1023);
    close(fd);

    cout << "Flushing files\n";
    dumpManager.flushAllFiles();  

    if (droppedPackets > 0) 
    {
	droppedPackets[readAmount-1] = 0;
	cout << "Dropped " << droppedPackets << " packets\n";
    }
    else
    {
	cout << "Could not read " << PROC_DROP_COUNT << endl;
    }

    close(fd);
    exit(0);
}
